/*************************************************
* MPI Addition/Subtraction Source File           *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/mp_core.h>

extern "C" {

/*************************************************
* Two Operand Addition                           *
*************************************************/
void bigint_add2(word* const x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   word carry = 0;

   for(u32bit j = 0; j != y_size; j++)
      {
      x[j] += y[j];
      word next_carry = (x[j] < y[j]) ? 1 : 0;
      x[j] += carry;
      carry = next_carry + (x[j] < carry) ? 1 : 0;
      }

   if(!carry) return;

   for(u32bit j = y_size; j != x_size; j++)
      {
      x[j]++;
      if(x[j]) return;
      }
   x[x_size]++;
   }

/*************************************************
* Three Operand Addition                         *
*************************************************/
void bigint_add3(word* const z,
                 const word* x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   if(x_size < y_size)
      {
      const word* t_ptr = x; x = y; y = t_ptr;
      u32bit t_size = x_size; x_size = y_size; y_size = t_size;
      }

   word carry = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      z[j] = x[j] + y[j];
      word next_carry = (z[j] < x[j]) ? 1 : 0;
      z[j] += carry;
      carry = next_carry + (z[j] < carry) ? 1 : 0;
      }

   for(u32bit j = y_size; j != x_size; j++)
      z[j] = x[j];

   if(!carry) return;

   for(u32bit j = y_size; j != x_size; j++)
      {
      z[j]++;
      if(z[j]) return;
      }
   z[x_size]++;
   }

/*************************************************
* Two Operand Subtraction                        *
*************************************************/
void bigint_sub2(word* const x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   word borrow = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      word temp = x[j] - y[j];
      word next_borrow = ((x[j] < temp) ? 1 : 0);
      x[j] = temp - borrow;
      borrow = next_borrow + ((x[j] == MP_WORD_MAX) ? borrow : 0);
      }

   if(!borrow) return;

   for(u32bit j = y_size; j != x_size; j++)
      {
      x[j]--;
      if(x[j] != MP_WORD_MAX) return;
      }
   }

/*************************************************
* Three Operand Subtraction                      *
*************************************************/
void bigint_sub3(word* const z,
                 const word* x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   word borrow = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      z[j] = x[j] - y[j];
      word next_borrow = ((x[j] < z[j]) ? 1 : 0);
      z[j] -= borrow;
      borrow = next_borrow + ((z[j] == MP_WORD_MAX) ? borrow : 0);
      }

   for(u32bit j = y_size; j != x_size; j++)
      z[j] = x[j];

   if(!borrow) return;

   for(u32bit j = y_size; j != x_size; j++)
      {
      z[j]--;
      if(z[j] != MP_WORD_MAX) return;
      }
   }

}
